"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createFakeData;
exports.getRandomNumber = getRandomNumber;
exports.randomImg = randomImg;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _invariant = _interopRequireDefault(require("invariant"));

var _faker = _interopRequireDefault(require("faker"));

var _utils = require("./utils");

/**
 * producing the immutable data with the given argument
 *
 * if object => recurse its children
 * if array => new List
 * if string => ''
 * if boolean => false
 * if number => 0
 * if toOne relation => null
 * if toMany relation => null
 * if dateTime => string
 */
// createFakeData = (schema: Object, listLength: number) => *
function createFakeData(root) {
  var listLength = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  if (!root) {
    return null;
  }

  var firstLevelKeys = getFirstLevelKeys(root);
  return root.hasOwnProperty('type') ? loop(root, root.keyName) : (0, _utils.mapSchema)(root, loop); // Loop schema

  function loop(schema, key) {
    var result;

    switch (schema.type) {
      case 'object':
        {
          if (schema.ui === 'editor') {
            result = {
              html: "<p>".concat(_faker.default.lorem.paragraphs(), "</p>")
            };
            break;
          }

          result = (0, _utils.mapSchema)(schema.items, loop);
          result.__typename = result.__typename || null;
          break;
        }

      case 'array':
        result = getArrayData(schema, key, listLength, loop);
        break;

      case 'number':
        result = _faker.default.random.number();
        break;

      case 'geoPoint':
        result = {
          __typename: null,
          lat: _faker.default.address.latitude(),
          lng: _faker.default.address.longitude(),
          address: _faker.default.address.city(),
          placeId: _faker.default.random.uuid()
        };
        break;

      case 'dateTime':
        result = _faker.default.date.past().toISOString();
        break;

      case 'file':
        result = {
          __typename: null,
          contentType: _faker.default.system.mimeType(),
          size: _faker.default.random.number(),
          name: _faker.default.system.commonFileName(),
          url: _faker.default.internet.url()
        };
        break;

      case 'image':
        result = {
          __typename: null,
          contentType: 'image/jpeg',
          size: _faker.default.random.number(),
          name: _faker.default.system.commonFileName(),
          url: randomImg()
        };
        break;

      case 'boolean':
        result = _faker.default.random.boolean();
        break;

      case 'string':
        {
          var lowerKeyName = schema.keyName && schema.keyName.toLowerCase() || '';

          if (lowerKeyName.indexOf('email') > -1) {
            result = _faker.default.internet.email();
          } else if (lowerKeyName.indexOf('phone') > -1) {
            result = _faker.default.phone.phoneNumber();
          } else if (schema.ui === "textarea") {
            result = _faker.default.lorem.paragraphs();
          } else {
            result = _faker.default.random.word();
          }

          break;
        }

      case 'enum':
        {
          if (schema.values) {
            result = schema.values[getRandomNumber(0, schema.values.length - 1)];
          } else {
            throw new Error('Enum type should have a values property.');
          }

          break;
        }

      case 'relation':
        {
          result = getRelation(schema, firstLevelKeys, listLength);
          break;
        }

      case 'json':
        {
          result = {};
          break;
        }

      default:
        (0, _invariant.default)(true, "unsupport type ".concat(schema.type));
        break;
    }

    return result;
  }
}

function getFirstLevelKeys(root) {
  if ('type' in root) {
    return root.items ? Object.keys(root.items) : {};
  }

  return Object.keys(root);
}

function getArrayData(schema, key, listLength, loop) {
  var result = [];

  for (var i = 0; i < listLength; ++i) {
    var item = void 0;

    if (schema.items && schema.items.items) {
      item = (0, _utils.mapSchema)(schema.items.items, loop);
    } else {
      item = schema.items.hasOwnProperty('type') ? loop(schema.items, schema.items.keyName) : (0, _utils.mapSchema)(schema.items, loop);
    }

    if ((0, _typeof2.default)(item) === 'object' && key) {
      item = (0, _objectSpread2.default)({}, item, {
        id: "".concat(key).concat(i + 1)
      });
    }

    result.push(item);
  }

  return result;
}

function getRelation(schema, firstLevelKeys, listLength) {
  var _schema$relation = schema.relation,
      type = _schema$relation.type,
      to = _schema$relation.to;
  var isInFirstLevel = Array.isArray(firstLevelKeys) && firstLevelKeys.indexOf(to) > -1 && listLength > 0;
  var result;

  switch (type) {
    case 'toMany':
      result = [];

      if (isInFirstLevel) {
        for (var i = 0; i < listLength; ++i) {
          if (Math.random() > 0.5 && to !== schema.keyName) {
            result.push("".concat(to).concat(i + 1));
          }
        }
      }

      break;

    case 'toOne':
    default:
      if (isInFirstLevel && to !== schema.keyName) {
        result = "".concat(to).concat(getRandomNumber(1, listLength));
      } else {
        result = null;
      }

      break;
  }

  return result;
}

function getRandomNumber(min, max) {
  return Math.floor(Math.random() * (max - min)) + min;
}

function randomImg() {
  return "https://placeimg.com/".concat(getRandomNumber(250, 350), "/").concat(getRandomNumber(300, 400), "/any");
}